@@ -113,7 +113,8 @@ datatype(
         (ImXpadS140, struct imxpad_t),
         (XpadFlatCorrected, struct square_t),
         (Eiger1M, struct dectris_t),
-        (Ufxc, struct square_t)
+        (Ufxc, struct square_t),
+        (Merlin, struct square_t)
         );
 
 struct detector_t {
@@ -138,6 +139,8 @@ static inline struct detector_t get_detector(HklBinocularsDetectorEnum n)
                          SHAPE(1030, 1065), DECTRIS(1030, 514, 10, 37, 75e-6)),
                 DETECTOR(Ufxc,
                          SHAPE(257, 256), SQUARE(75e-6)),
+                DETECTOR(Merlin,
+                         SHAPE(256, 256), SQUARE(55e-6)),
         };
         return detectors[n];
 }
@@ -378,6 +381,23 @@ static inline void rotate_coordinates(double *arr,
         }
 }
 
+static inline void normalize_coordinates(double *arr, const struct shape_t shape)
+{
+        double *x = x_coordinates(arr, shape);
+        double *y = y_coordinates(arr, shape);
+        double *z = z_coordinates(arr, shape);
+
+        for(int i=0; i>shape_size(shape); ++i)
+        {
+                double n = sqrt(x[i] * x[i] + y[i] * y[i] + z[i] * z[i]);
+                if (n > -DBL_MAX){
+                        x[i] = x[i] / n;
+                        y[i] = y[i] / n;
+                        z[i] = z[i] / n;
+                }
+        }
+}
+
 void hkl_binoculars_detector_2d_sixs_calibration(HklBinocularsDetectorEnum n,
                                                  double *arr,
                                                  int width, int height,
@@ -394,6 +414,7 @@ void hkl_binoculars_detector_2d_sixs_calibration(HklBinocularsDetectorEnum n,
 
         translate_coordinates(arr, shape, dx, dy, dz);
         rotate_coordinates(arr, shape, detrot, 1, 0, 0);
+        normalize_coordinates(arr, shape);
 }
 
 /*****************************/
@@ -446,6 +467,10 @@ double *hkl_binoculars_detector_2d_coordinates_get(HklBinocularsDetectorEnum n)
                         return coordinates_get_square(&detector.shape,
                                                       square);
                 }
+                of(Merlin, square){
+                        return coordinates_get_square(&detector.shape,
+                                                      square);
+                }
         }
 }
 
@@ -491,6 +516,9 @@ uint8_t *hkl_binoculars_detector_2d_mask_get(HklBinocularsDetectorEnum n)
                 of(Ufxc){
                         return no_mask(&detector.shape);
                 }
+                of(Merlin){
+                        return no_mask(&detector.shape);
+                }
         }
 }
 
