@@ -96,7 +96,7 @@ double *hkl_binoculars_axis_array(const HklBinocularsAxis *self)
 static inline int hkl_binoculars_axis_contains_axis(const HklBinocularsAxis *self,
                                                     const HklBinocularsAxis *other)
 {
-        return self->imin >= other->imin && self->imax <= other->imax;
+        return self->imin <= other->imin && self->imax >= other->imax;
 }
 
 static inline void hkl_binoculars_axis_merge(HklBinocularsAxis *self, const HklBinocularsAxis *other)
@@ -127,7 +127,7 @@ static inline void merge_axes(darray_axis *axes,
 }
 
 static inline int does_not_include(const darray_axis *axes,
-                                  const darray_axis *others)
+                                   const darray_axis *others)
 {
         size_t i;
         int res = 0;
@@ -187,6 +187,12 @@ void hkl_binoculars_space_fprintf(FILE *f, const HklBinocularsSpace *self)
         fprintf(f, "\nmasked pixels: %ld (%f%%)", masked, (double)masked / self->max_items * 100);
 }
 
+static inline void hkl_binoculars_space_item_fprintf(FILE *f, const HklBinocularsSpaceItem *self)
+{
+        fprintf(f, "item->indexes(%p) v: %ld %ld %ld, intensity: %d", &self->indexes_0[0],
+                self->indexes_0[0], self->indexes_0[1], self->indexes_0[2], self->intensity);
+}
+
 static inline void space_update_axes(HklBinocularsSpace *space,
                                      const char *names[],
                                      size_t n_pixels,
@@ -215,129 +221,125 @@ static inline void space_update_axes(HklBinocularsSpace *space,
 }
 
 /* the array is pre filled with the pixel coordinates */
-void hkl_binoculars_space_q(HklBinocularsSpace *space,
-                            const HklGeometry *geometry,
-                            const uint16_t *image,
-                            size_t n_pixels,
-                            double weight,
-                            const double *pixels_coordinates,
-                            size_t pixels_coordinates_ndim,
-                            const size_t *pixels_coordinates_dims,
-                            const double *resolutions,
-                            size_t n_resolutions,
-                            const uint8_t *masked)
-{
-	size_t i, j;
-	const char * names[] = {"qx", "qy", "qz"};
 
-        assert(ARRAY_SIZE(names) == darray_size(space->axes));
-        assert(ARRAY_SIZE(names) == n_resolutions);
-        assert(n_pixels == space->max_items);
+#define HKL_BINOCULARS_SPACE_Q_IMPL(image_t)                            \
+        HKL_BINOCULARS_SPACE_Q_DECL(image_t)                            \
+        {                                                               \
+                size_t i, j;                                            \
+                const char * names[] = {"qx", "qy", "qz"};              \
+                                                                        \
+                assert(ARRAY_SIZE(names) == darray_size(space->axes));  \
+                assert(ARRAY_SIZE(names) == n_resolutions);             \
+                assert(n_pixels == space->max_items);                   \
+                                                                        \
+                const double *q_x = &pixels_coordinates[0 * n_pixels];  \
+                const double *q_y = &pixels_coordinates[1 * n_pixels];  \
+                const double *q_z = &pixels_coordinates[2 * n_pixels];  \
+                                                                        \
+                HklSample *sample = hkl_sample_new("test");             \
+                HklDetector *detector = hkl_detector_factory_new(HKL_DETECTOR_TYPE_0D); \
+                const HklQuaternion q = hkl_geometry_detector_rotation_get(geometry, detector); \
+                const HklVector ki = hkl_geometry_ki_get(geometry);     \
+                double k = hkl_vector_norm2(&ki);                       \
+                HklQuaternion qs_1 = hkl_geometry_sample_rotation_get(geometry, sample); \
+                switch(surf){                                           \
+                case HKL_BINOCULARS_SURFACE_ORIENTATION_VERTICAL:       \
+                {                                                       \
+                        HklQuaternion q_ub = {{0, -1, 0, 0}};           \
+                        hkl_quaternion_times_quaternion(&qs_1, &q_ub);  \
+                        break;                                          \
+                };                                                      \
+                case HKL_BINOCULARS_SURFACE_ORIENTATION_HORIZONTAL:     \
+                case HKL_BINOCULARS_SURFACE_ORIENTATION_NUM_ORIENTATION:\
+                        break;                                          \
+                };                                                      \
+                hkl_quaternion_conjugate(&qs_1);                        \
+                                                                        \
+                darray_size(space->items) = 0;                          \
+                                                                        \
+                for(i=0;i<n_pixels;++i){                                \
+                        if(NULL == masked || 0 == masked[i]){           \
+                                HklBinocularsSpaceItem item;            \
+                                HklVector v = {{q_x[i], q_y[i], q_z[i]}}; \
+                                                                        \
+                                hkl_vector_rotated_quaternion(&v, &q);  \
+                                hkl_vector_times_double(&v, k);         \
+                                hkl_vector_minus_vector(&v, &ki);       \
+                                hkl_vector_rotated_quaternion(&v, &qs_1); \
+                                                                        \
+                                for(j=0; j<ARRAY_SIZE(names); ++j){     \
+                                        item.indexes_0[j] = rint(v.data[j] / resolutions[j] / 10); \
+                                }                                       \
+                                item.intensity = rint((double)image[i] * weight); \
+                                                                        \
+                                                                        \
+                                darray_append(space->items, item);      \
+                        }                                               \
+                }                                                       \
+                                                                        \
+                space_update_axes(space, names, n_pixels, resolutions); \
+                                                                        \
+                hkl_detector_free(detector);                            \
+                hkl_sample_free(sample);                                \
+        }
 
-	const double *q_x = &pixels_coordinates[0 * n_pixels];
-	const double *q_y = &pixels_coordinates[1 * n_pixels];
-	const double *q_z = &pixels_coordinates[2 * n_pixels];
+HKL_BINOCULARS_SPACE_Q_IMPL(int32_t);
+HKL_BINOCULARS_SPACE_Q_IMPL(uint16_t);
+HKL_BINOCULARS_SPACE_Q_IMPL(uint32_t);
 
-	HklSample *sample = hkl_sample_new("test");
-	HklDetector *detector = hkl_detector_factory_new(HKL_DETECTOR_TYPE_0D);
-	const HklQuaternion q = hkl_geometry_detector_rotation_get(geometry, detector);
-	HklQuaternion qs_1 = hkl_geometry_sample_rotation_get(geometry, sample);
-	const HklVector ki = hkl_geometry_ki_get(geometry);
-        double k = hkl_vector_norm2(&ki);
-	hkl_quaternion_conjugate(&qs_1);
+#define HKL_BINOCULARS_SPACE_HKL_IMPL(image_t)                          \
+        HKL_BINOCULARS_SPACE_HKL_DECL(image_t)                          \
+        {                                                               \
+                size_t i, j;                                            \
+                const char * names[] = {"H", "K", "L"};                 \
+                                                                        \
+                assert(ARRAY_SIZE(names) == darray_size(space->axes));  \
+                assert(ARRAY_SIZE(names) == n_resolutions);             \
+                assert(n_pixels == space->max_items);                   \
+                                                                        \
+                const double *h = &pixels_coordinates[0 * n_pixels];    \
+                const double *k = &pixels_coordinates[1 * n_pixels];    \
+                const double *l = &pixels_coordinates[2 * n_pixels];    \
+                                                                        \
+                HklDetector *detector = hkl_detector_factory_new(HKL_DETECTOR_TYPE_0D); \
+                const HklQuaternion q_d = hkl_geometry_detector_rotation_get(geometry, detector); \
+                HklQuaternion qs = hkl_geometry_sample_rotation_get(geometry, sample); \
+                const HklVector ki = hkl_geometry_ki_get(geometry);     \
+                double K = hkl_vector_norm2(&ki);                       \
+                HklMatrix RUB;                                          \
+                HklMatrix RUB_1;                                        \
+                hkl_quaternion_to_matrix(&qs, &RUB);                    \
+                hkl_matrix_times_matrix(&RUB, hkl_sample_UB_get(sample)); \
+                hkl_matrix_inv(&RUB, &RUB_1);                           \
+                                                                        \
+                darray_size(space->items) = 0;                          \
+                for(i=0;i<n_pixels;++i){                                \
+                        if(NULL == masked || 0 == masked[i]){           \
+                                HklBinocularsSpaceItem item;            \
+                                HklVector v = {{h[i], k[i], l[i]}};     \
+                                                                        \
+                                hkl_vector_rotated_quaternion(&v, &q_d); \
+                                hkl_vector_times_double(&v, K);         \
+                                hkl_vector_minus_vector(&v, &ki);       \
+                                hkl_matrix_times_vector(&RUB_1, &v);    \
+                                                                        \
+                                for(j=0; j<ARRAY_SIZE(names); ++j){     \
+                                        item.indexes_0[j] = rint(v.data[j] / resolutions[j]); \
+                                }\
+                                item.intensity = rint((double)image[i] * weight); \
+                                                                        \
+                                darray_append(space->items, item);      \
+                        }                                               \
+                }                                                       \
+                                                                        \
+                space_update_axes(space, names, n_pixels, resolutions); \
+                                                                        \
+                hkl_detector_free(detector);                            \
+        }
 
-	/* compute the coordinates in the last axis basis and the
-	 * indexes */
-        darray_size(space->items) = 0;
-
-	for(i=0;i<n_pixels;++i){
-                if(NULL == masked || 0 == masked[i]){
-                        HklBinocularsSpaceItem item;
-                        HklVector v = {{q_x[i], q_y[i], q_z[i]}};
-
-                        hkl_vector_rotated_quaternion(&v, &q);
-                        hkl_vector_normalize(&v);
-                        hkl_vector_times_double(&v, k);
-                        hkl_vector_minus_vector(&v, &ki);
-                        hkl_vector_rotated_quaternion(&v, &qs_1);
-
-                        for(j=0; j<ARRAY_SIZE(names); ++j)
-                                item.indexes_0[j] = rint(v.data[j] / resolutions[j]);
-                        item.intensity = rint((double)image[i] * weight);
-
-                        darray_append(space->items, item);
-                }
-	}
-
-        space_update_axes(space, names, n_pixels, resolutions);
-
-	hkl_detector_free(detector);
-        hkl_sample_free(sample);
-}
-
-/* the array is pre filled with the pixel coordinates */
-void hkl_binoculars_space_hkl(HklBinocularsSpace *space,
-                              const HklGeometry *geometry,
-                              const HklSample *sample,
-                              const uint16_t *image,
-                              size_t n_pixels,
-                              double weight,
-                              const double *pixels_coordinates,
-                              size_t pixels_coordinates_ndim,
-                              const size_t *pixels_coordinates_dims,
-                              const double *resolutions,
-                              size_t n_resolutions,
-                              const uint8_t *masked)
-{
-	size_t i, j;
-	const char * names[] = {"H", "K", "L"};
-
-        assert(ARRAY_SIZE(names) == darray_size(space->axes));
-        assert(ARRAY_SIZE(names) == n_resolutions);
-        assert(n_pixels == space->max_items);
-
-        const double *h = &pixels_coordinates[0 * n_pixels];
-	const double *k = &pixels_coordinates[1 * n_pixels];
-	const double *l = &pixels_coordinates[2 * n_pixels];
-
-	HklDetector *detector = hkl_detector_factory_new(HKL_DETECTOR_TYPE_0D);
-	const HklQuaternion q_d = hkl_geometry_detector_rotation_get(geometry, detector);
-	HklQuaternion qs = hkl_geometry_sample_rotation_get(geometry, sample);
-	const HklVector ki = hkl_geometry_ki_get(geometry);
-        double K = hkl_vector_norm2(&ki);
-        HklMatrix RUB;
-        HklMatrix RUB_1;
-        hkl_quaternion_to_matrix(&qs, &RUB);
-	hkl_matrix_times_matrix(&RUB, hkl_sample_UB_get(sample));
-        hkl_matrix_inv(&RUB, &RUB_1);
-
-	/* compute the coordinates in the last axis basis and the
-	 * indexes */
-        darray_size(space->items) = 0;
-	for(i=0;i<n_pixels;++i){
-                if(NULL == masked || 0 == masked[i]){
-                        HklBinocularsSpaceItem item;
-                        HklVector v = {{h[i], k[i], l[i]}};
-
-                        hkl_vector_rotated_quaternion(&v, &q_d);
-                        hkl_vector_normalize(&v);
-                        hkl_vector_times_double(&v, K);
-                        hkl_vector_minus_vector(&v, &ki);
-                        hkl_matrix_times_vector(&RUB_1, &v);
-
-                        for(j=0; j<ARRAY_SIZE(names); ++j)
-                                item.indexes_0[j] = rint(v.data[j] / resolutions[j]);
-                        item.intensity = rint((double)image[i] * weight);
-
-                        darray_append(space->items, item);
-                }
-	}
-
-        space_update_axes(space, names, n_pixels, resolutions);
-
-        /* hkl_binoculars_space_fprintf(stdout, space); */
-	hkl_detector_free(detector);
-}
+HKL_BINOCULARS_SPACE_HKL_IMPL(int32_t);
+HKL_BINOCULARS_SPACE_HKL_IMPL(uint16_t);
+HKL_BINOCULARS_SPACE_HKL_IMPL(uint32_t);
 
 /* Cube */
 
@@ -508,6 +510,16 @@ HklBinocularsCube *hkl_binoculars_cube_new_empty(void)
         return empty_cube(NULL);
 }
 
+HklBinocularsCube *hkl_binoculars_cube_new_empty_from_cube(const HklBinocularsCube *cube)
+{
+	HklBinocularsCube *self = empty_cube(&cube->axes);
+
+	/* allocated the final cube */
+        calloc_cube(self);
+
+	return self;
+}
+
 HklBinocularsCube *hkl_binoculars_cube_new_from_space(const HklBinocularsSpace *space)
 {
 	HklBinocularsCube *self = empty_cube(&space->axes);
@@ -617,13 +629,25 @@ void hkl_binoculars_cube_add_space(HklBinocularsCube *self,
 {
         HklBinocularsCube *cube;
 
+#ifdef DEBUG
+        fprintf(stdout, "\nENTERING hkl_binoculars_cube_add_space:\n");
+        hkl_binoculars_cube_fprintf(stdout, self);
+        hkl_binoculars_space_fprintf(stdout, space);
+#endif
         if(is_empty(self)){
+#ifdef DEBUG
+                fprintf(stdout, "\nCreate a new empty cube");
+#endif
+
                 cube = hkl_binoculars_cube_new_from_space(space);
                 switch_content(self, cube);
                 hkl_binoculars_cube_free(cube);
         }else{
                 /* check the compatibility of the cube and the space. */
                 if (does_not_include(&self->axes, &space->axes)){
+#ifdef DEBUG
+                        fprintf(stdout, "\nthe Cube does not contain the space, so create a new cube.");
+#endif
                         HklBinocularsCube *cube = empty_cube(&self->axes);
 
                         merge_axes(&cube->axes, &space->axes); /* circonscript */
@@ -637,4 +661,9 @@ void hkl_binoculars_cube_add_space(HklBinocularsCube *self,
                 }
                 add_space(self, space);
         }
+#ifdef DEBUG
+        fprintf(stdout, "\n");
+        hkl_binoculars_cube_fprintf(stdout, self);
+        fprintf(stdout, "\nLEAVING hkl_binoculars_cube_add_space:\n");
+#endif
 }
