@@ -38,6 +38,7 @@ typedef enum _HklBinocularsDetectorEnum
         HKL_BINOCULARS_DETECTOR_IMXPAD_S70,
         HKL_BINOCULARS_DETECTOR_DECTRIS_EIGER1M,
         HKL_BINOCULARS_DETECTOR_UFXC,
+        HKL_BINOCULARS_DETECTOR_MERLIN,
         /* Add new your detectors here */
         HKL_BINOCULARS_DETECTOR_NUM_DETECTORS,
 } HklBinocularsDetectorEnum;
@@ -107,35 +108,56 @@ struct _HklBinocularsSpace
         darray_HklBinocularsSpaceItem items;
 };
 
+typedef enum _HklBinocularsSurfaceOrientationEnum
+{
+        HKL_BINOCULARS_SURFACE_ORIENTATION_VERTICAL = 0,
+        HKL_BINOCULARS_SURFACE_ORIENTATION_HORIZONTAL,
+        /* Add new your detectors here */
+        HKL_BINOCULARS_SURFACE_ORIENTATION_NUM_ORIENTATION,
+} HklBinocularsSurfaceOrientationEnum;
+
+
 HKLAPI extern HklBinocularsSpace *hkl_binoculars_space_new(size_t n_indexes_0,
                                                            size_t n_axes);
 
 HKLAPI extern void hkl_binoculars_space_free(HklBinocularsSpace *self);
 
-HKLAPI extern void hkl_binoculars_space_q(HklBinocularsSpace *self,
-                                          const HklGeometry *geometry,
-                                          const uint16_t *image,
-                                          size_t n_pixels,
-                                          double weight,
-                                          const double *pixels_coordinates,
-                                          size_t pixels_coordinates_ndim,
-                                          const size_t *pixels_coordinates_dims,
-                                          const double *resolutions,
-                                          size_t n_resolutions,
-                                          const uint8_t *mask);
+#define HKL_BINOCULARS_SPACE_Q_DECL(image_t)                            \
+        void hkl_binoculars_space_q_ ## image_t (HklBinocularsSpace *space, \
+                                                 const HklGeometry *geometry, \
+                                                 const image_t *image,  \
+                                                 size_t n_pixels,       \
+                                                 double weight,         \
+                                                 const double *pixels_coordinates, \
+                                                 size_t pixels_coordinates_ndim, \
+                                                 const size_t *pixels_coordinates_dims, \
+                                                 const double *resolutions, \
+                                                 size_t n_resolutions,  \
+                                                 const uint8_t *masked, \
+                                                 HklBinocularsSurfaceOrientationEnum surf)
 
-HKLAPI extern void hkl_binoculars_space_hkl(HklBinocularsSpace *self,
-                                            const HklGeometry *geometry,
-                                            const HklSample *sample,
-                                            const uint16_t *image,
-                                            size_t n_pixels,
-                                            double weight,
-                                            const double *pixels_coordinates,
-                                            size_t pixels_coordinates_ndim,
-                                            const size_t *pixels_coordinates_dims,
-                                            const double *resolutions,
-                                            size_t n_resolutions,
-                                            const uint8_t *mask);
+HKLAPI extern HKL_BINOCULARS_SPACE_Q_DECL(int32_t);
+HKLAPI extern HKL_BINOCULARS_SPACE_Q_DECL(uint16_t);
+HKLAPI extern HKL_BINOCULARS_SPACE_Q_DECL(uint32_t);
+
+
+#define HKL_BINOCULARS_SPACE_HKL_DECL(image_t)\
+        void hkl_binoculars_space_hkl_ ## image_t (HklBinocularsSpace *space,\
+                                                   const HklGeometry *geometry, \
+                                                   const HklSample *sample, \
+                                                   const image_t *image, \
+                                                   size_t n_pixels,     \
+                                                   double weight,       \
+                                                   const double *pixels_coordinates, \
+                                                   size_t pixels_coordinates_ndim, \
+                                                   const size_t *pixels_coordinates_dims, \
+                                                   const double *resolutions, \
+                                                   size_t n_resolutions, \
+                                                   const uint8_t *masked)
+
+HKLAPI extern HKL_BINOCULARS_SPACE_HKL_DECL(int32_t);
+HKLAPI extern HKL_BINOCULARS_SPACE_HKL_DECL(uint16_t);
+HKLAPI extern HKL_BINOCULARS_SPACE_HKL_DECL(uint32_t);
 
 /********/
 /* Cube */
@@ -157,6 +179,8 @@ HKLAPI extern HklBinocularsCube *hkl_binoculars_cube_new(size_t n_spaces,
 
 HKLAPI extern HklBinocularsCube *hkl_binoculars_cube_new_empty(void);
 
+HKLAPI extern HklBinocularsCube *hkl_binoculars_cube_new_empty_from_cube(const HklBinocularsCube *src);
+
 HKLAPI extern HklBinocularsCube *hkl_binoculars_cube_new_copy(const HklBinocularsCube *src);
 
 HKLAPI extern HklBinocularsCube *hkl_binoculars_cube_new_from_space(const HklBinocularsSpace *space);
