@@ -14,12 +14,12 @@
 -}
 module Hkl.Binoculars.Common
   ( Chunk(..)
-  , InputFn(..)
   , DataFrameSpace(..)
+  , InputFn(..)
   , addSpace
   , chunk
+  , clength
   , mkCube'
-  , mkJobs'
   , toList
   , withCubeAccumulator
   ) where
@@ -27,7 +27,6 @@ module Hkl.Binoculars.Common
 import           Control.Exception     (bracket)
 import           Data.Array.Repa       (Shape)
 import           Data.IORef            (IORef, newIORef, readIORef)
-import           Data.Word             (Word16)
 import           Foreign.ForeignPtr    (ForeignPtr, withForeignPtr)
 import           Foreign.Marshal.Array (withArrayLen)
 import           Foreign.Ptr           (Ptr)
@@ -36,11 +35,16 @@ import           Path                  (Abs, File, Path, fromAbsFile)
 import           Text.Printf           (printf)
 
 import           Hkl.C.Binoculars
+import           Hkl.Image
 import           Hkl.Orphan            ()
 
 data Chunk n a = Chunk !a !n !n
 deriving instance (Show n, Show a) => Show (Chunk n a)
 
+clength :: Num n => Chunk n a -> n
+clength (Chunk _ l h) = h - l + 1
+{-# SPECIALIZE clength :: Chunk Int FilePath -> Int  #-}
+
 cweight :: Num n => Chunk n a -> n
 cweight (Chunk _ l h) = h - l
 
@@ -55,7 +59,7 @@ chunk target = go target target
     go tgt gap [x]     = golast tgt gap x
     go tgt gap ~(x:xs) =
       let gap' = gap - cweight x
-      in if | gap' > 0                -> cons1 x $ go tgt gap' xs
+      in if | gap' > 0                 -> cons1 x $ go tgt gap' xs
             | gap' == 0                -> [x] : go tgt tgt xs
             | (x1, x2) <- csplit x gap -> [x1] : go tgt tgt (x2 : xs)
 
@@ -72,12 +76,9 @@ toList (InputFn f)           = [f]
 toList (InputRange tmpl f t) = [printf tmpl i | i <- [f..t]]
 toList (InputList fs)        = map fromAbsFile fs
 
-mkJobs' :: Int -> [FilePath] -> [Int] -> [[Chunk Int FilePath]]
-mkJobs' n fns ts = chunk n [Chunk f 0 t | (f, t) <- zip fns ts]
-
 --  DataFrameSpace
 
-data DataFrameSpace sh = DataFrameSpace (ForeignPtr Word16) (Space sh) Double
+data DataFrameSpace sh = DataFrameSpace Image (Space sh) Double
   deriving Show
 
 --  Create the Cube
@@ -97,7 +98,7 @@ mkCube' dfs = do
     peek =<< {-# SCC "hkl_binoculars_cube_new'" #-} hkl_binoculars_cube_new' (toEnum nSpaces') spaces'
 
 {-# INLINE addSpace #-}
-addSpace :: Shape sh => DataFrameSpace sh -> (Cube' sh) -> IO (ForeignPtr (Cube' sh))
+addSpace :: Shape sh => DataFrameSpace sh -> Cube' sh -> IO (ForeignPtr (Cube' sh))
 addSpace df EmptyCube' = do
   (Cube' fp) <- mkCube' [df]
   return fp
@@ -114,7 +115,13 @@ data InputFn = InputFn FilePath
              | InputList [Path Abs File]
   deriving Show
 
-withCubeAccumulator :: Shape sh => (IORef (Cube' sh)  -> IO ()) -> IO (Cube' sh)
-withCubeAccumulator f = bracket (newIORef =<< peek =<< hkl_binoculars_cube_new_empty') pure (\r -> f r >> readIORef r)
+withCubeAccumulator :: Shape sh => Cube' sh -> (IORef (Cube' sh)  -> IO ()) -> IO (Cube' sh)
+withCubeAccumulator c f = bracket
+  (newIORef =<< peek =<< case c of
+                           EmptyCube' -> hkl_binoculars_cube_new_empty'
+                           (Cube' fp) -> withForeignPtr fp $ \p -> hkl_binoculars_cube_new_empty_from_cube' p
+  )
+  pure
+  (\r -> f r >> readIORef r)
 
 -- Projections
