@@ -19,8 +19,10 @@ module Hkl.Binoculars.Config
     ( BinocularsConfig(..)
     , ConfigRange(..)
     , DestinationTmpl(..)
+    , InputRange(..)
     , InputType(..)
     , ProjectionType(..)
+    , SurfaceOrientation(..)
     , configRangeP
     , combineWithCmdLineArgs
     , destination'
@@ -35,6 +37,7 @@ module Hkl.Binoculars.Config
     ) where
 
 
+import           Control.Applicative               (many, (<|>))
 import           Control.Lens                      (makeLenses, (^.))
 import           Control.Monad.Catch               (Exception, MonadThrow,
                                                     throwM)
@@ -42,11 +45,9 @@ import           Control.Monad.Catch.Pure          (runCatch)
 import           Control.Monad.IO.Class            (MonadIO, liftIO)
 import           Control.Monad.Logger              (MonadLogger)
 import           Data.Array.Repa.Index             (DIM2)
-#if MIN_VERSION_extra(1, 6, 14)
-import           Data.Either.Extra                 (mapLeft, mapRight)
-#endif
 import           Data.Attoparsec.Text              (Parser, char, decimal,
-                                                    sepBy)
+                                                    parseOnly, satisfy, sepBy)
+import           Data.Either.Extra                 (mapLeft, mapRight)
 import           Data.Ini.Config.Bidir             (FieldValue (..), IniSpec,
                                                     bool, field, getIniValue,
                                                     ini, listWithSeparator,
@@ -58,7 +59,8 @@ import           Data.Maybe                        (fromMaybe)
 import           Data.Text                         (Text, breakOn, drop,
                                                     findIndex, length, lines,
                                                     pack, replace, strip, take,
-                                                    takeWhile, unlines, unpack)
+                                                    takeWhile, toLower, unlines,
+                                                    unpack, unwords)
 import           Data.Text.IO                      (putStr, readFile)
 import           Data.Typeable                     (Typeable)
 import           GHC.Exts                          (IsList)
@@ -74,37 +76,16 @@ import           Text.Printf                       (printf)
 
 import           Prelude                           hiding (drop, length, lines,
                                                     putStr, readFile, take,
-                                                    takeWhile, unlines)
+                                                    takeWhile, unlines, unwords)
 
 import           Hkl.Detector
 import           Hkl.Types
 import           Paths_hkl
 
 
-#if MIN_VERSION_extra(1, 6, 14)
-#else
--- for buster
-
---  The 'mapLeft' function takes a function and applies it to an Either value
--- iff the value takes the form @'Left' _@.
---
--- > mapLeft show (Left 1) == Left "1"
--- > mapLeft show (Right True) == Right True
-mapLeft :: (a -> c) -> Either a b -> Either c b
-mapLeft f = either (Left . f) Right
-
---  The 'mapRight' function takes a function and applies it to an Either value
--- iff the value takes the form @'Right' _@.
---
--- > mapRight show (Left 1) == Left 1
--- > mapRight show (Right True) == Right "True"
-mapRight :: (b -> c) -> Either a b -> Either a c
-mapRight = fmap
-#endif
-
 data HklBinocularsConfigException = NoFilesInTheGivenDirectory (Path Abs Dir)
                                   | NoDataFilesInTheGivenDirectory (Path Abs Dir)
-                                  | NoFilesInRangeInTheGivenDirectory (Path Abs Dir) (ConfigRange Int)
+                                  | NoFilesInRangeInTheGivenDirectory (Path Abs Dir) ConfigRange
                                   | ResolutionNotCompatibleWithProjectionNbOfCoordinates [Double] Int
     deriving (Show)
 instance Exception HklBinocularsConfigException
@@ -113,7 +94,11 @@ newtype DestinationTmpl =
   DestinationTmpl { unDestinationTmpl :: Text }
   deriving (Eq, Show)
 
+newtype InputTmpl = InputTmpl { unInputTmpl :: Text }
+  deriving (Eq, Show)
+
 data InputType = CristalK6C
+               | MarsFlyscan
                | SixsFlyMedH
                | SixsFlyMedV
                | SixsFlyMedVEiger
@@ -127,9 +112,25 @@ data InputType = CristalK6C
                | SixsSbsMedVFixDetector
   deriving (Eq, Show)
 
-newtype ConfigRange a = ConfigRange [a]
+data InputRange = InputRangeSingle Int
+                | InputRangeFromTo Int Int
+                deriving (Eq, Show)
+
+newtype ConfigRange = ConfigRange [InputRange]
   deriving (Eq, Show, IsList)
 
+data SurfaceOrientation = SurfaceOrientationVertical
+                        | SurfaceOrientationHorizontal
+  deriving (Eq, Show)
+
+instance Enum SurfaceOrientation where
+  fromEnum SurfaceOrientationVertical   = 0
+  fromEnum SurfaceOrientationHorizontal = 1
+
+  toEnum 0         = SurfaceOrientationVertical
+  toEnum 1         = SurfaceOrientationHorizontal
+  toEnum unmatched = error ("Key.toEnum: Cannot match " ++ show unmatched)
+
 data ProjectionType = QxQyQzProjection
                     | HklProjection
   deriving (Eq, Show)
@@ -140,12 +141,14 @@ data BinocularsConfig = BinocularsConfig
   , _binocularsDispatcherOverwrite         :: Bool
   , _binocularsInputItype                  :: InputType
   , _binocularsInputNexusdir               :: Maybe (Path Abs Dir)
-  , _binocularsInputInputRange             :: Maybe (ConfigRange Int)
+  , _binocularsInputTmpl                   :: Maybe InputTmpl
+  , _binocularsInputInputRange             :: Maybe ConfigRange
   , _binocularsInputDetector               :: Maybe (Detector Hkl DIM2)
   , _binocularsInputCentralpixel           :: (Int, Int)
   , _binocularsInputSdd                    :: Length Double
   , _binocularsInputDetrot                 :: Maybe (Angle Double)
   , _binocularsInputAttenuationCoefficient :: Maybe Double
+  , _binocularsInputSurfaceOrientation     :: Maybe SurfaceOrientation
   , _binocularsInputMaskmatrix             :: Maybe Text
   , _binocularsInputA                      :: Maybe (Length Double)
   , _binocularsInputB                      :: Maybe (Length Double)
@@ -156,6 +159,7 @@ data BinocularsConfig = BinocularsConfig
   , _binocularsInputUx                     :: Maybe (Angle Double)
   , _binocularsInputUy                     :: Maybe (Angle Double)
   , _binocularsInputUz                     :: Maybe (Angle Double)
+  , _binocularsInputWavelength             :: Maybe (Length Double)
   , _binocularsProjectionPtype             :: ProjectionType
   , _binocularsProjectionResolution        :: [Double]
   } deriving (Eq, Show)
@@ -169,12 +173,14 @@ binocularsConfigDefault = BinocularsConfig
   , _binocularsDispatcherOverwrite = False
   , _binocularsInputItype = SixsFlyScanUhv
   , _binocularsInputNexusdir = Nothing
+  , _binocularsInputTmpl = Nothing
   , _binocularsInputInputRange = Nothing
   , _binocularsInputDetector = Nothing
   , _binocularsInputCentralpixel = (0, 0)
   , _binocularsInputSdd = 1 *~ meter
   , _binocularsInputDetrot = Nothing
   , _binocularsInputAttenuationCoefficient = Nothing
+  , _binocularsInputSurfaceOrientation = Just SurfaceOrientationVertical
   , _binocularsInputMaskmatrix = Nothing
   , _binocularsInputA = Nothing
   , _binocularsInputB = Nothing
@@ -185,6 +191,7 @@ binocularsConfigDefault = BinocularsConfig
   , _binocularsInputUx = Nothing
   , _binocularsInputUy = Nothing
   , _binocularsInputUz = Nothing
+  , _binocularsInputWavelength = Nothing
   , _binocularsProjectionPtype = QxQyQzProjection
   , _binocularsProjectionResolution = [0.01, 0.01, 0.01]
   }
@@ -207,6 +214,15 @@ destinationTmpl = FieldValue { fvParse = parse, fvEmit = emit }
       emit :: DestinationTmpl -> Text
       emit (DestinationTmpl t) = t
 
+inputTmpl :: FieldValue InputTmpl
+inputTmpl = FieldValue { fvParse = parse, fvEmit = emit }
+    where
+      parse :: Text -> Either String InputTmpl
+      parse = Right . InputTmpl . uncomment
+
+      emit :: InputTmpl -> Text
+      emit (InputTmpl t) = t
+
 binocularsConfigSpec :: IniSpec BinocularsConfig ()
 binocularsConfigSpec = do
   section "dispatcher" $ do
@@ -216,12 +232,14 @@ binocularsConfigSpec = do
   section "input" $ do
     binocularsInputItype .= field "type" inputType
     binocularsInputNexusdir .=? field "nexusdir" pathAbsDir
+    binocularsInputTmpl .=? field "inputtmpl" inputTmpl
     binocularsInputInputRange .=? field "inputrange" configRange
     binocularsInputDetector .=? field "detector" detector
     binocularsInputCentralpixel .= field "centralpixel" centralPixel
     binocularsInputSdd .= field "sdd" (numberUnit meter)
     binocularsInputDetrot .=? field "detrot" (numberUnit degree)
     binocularsInputAttenuationCoefficient .=? field "attenuation_coefficient" number'
+    binocularsInputSurfaceOrientation .=? field "surface_orientation" surfaceOrientation
     binocularsInputMaskmatrix .=? field "maskmatrix" text
     binocularsInputA .=? field "a" (numberUnit angstrom)
     binocularsInputB .=? field "b" (numberUnit angstrom)
@@ -232,6 +250,7 @@ binocularsConfigSpec = do
     binocularsInputUx .=? field "ux" (numberUnit degree)
     binocularsInputUy .=? field "uy" (numberUnit degree)
     binocularsInputUz .=? field "uz" (numberUnit degree)
+    binocularsInputWavelength .=? field "wavelength" (numberUnit angstrom)
   section "projection" $ do
     binocularsProjectionPtype .= field "type" projectionType
     binocularsProjectionResolution .= field "resolution" (listWithSeparator "," number')
@@ -241,7 +260,7 @@ inputType = FieldValue { fvParse = parse . strip. uncomment, fvEmit = emit }
     where
       parse :: Text -> Either String InputType
       parse t
-          | t == emit SixsFlyMedH = Right SixsFlyMedH
+          | toLower t == emit SixsFlyMedH = Right SixsFlyMedH
           | t == emit SixsFlyMedV = Right SixsFlyMedV
           | t == emit SixsFlyMedVEiger = Right SixsFlyMedVEiger
           | t == emit SixsFlyMedVS70 = Right SixsFlyMedVS70
@@ -253,6 +272,7 @@ inputType = FieldValue { fvParse = parse . strip. uncomment, fvEmit = emit }
           | t == emit SixsSbsMedV = Right SixsSbsMedV
           | t == emit SixsSbsMedVFixDetector = Right SixsSbsMedVFixDetector
           | t == emit CristalK6C = Right CristalK6C
+          | t == emit MarsFlyscan = Right MarsFlyscan
           | otherwise = Left ("Unsupported \"" ++ unpack t ++ "\" input format")
 
       emit :: InputType -> Text
@@ -268,6 +288,21 @@ inputType = FieldValue { fvParse = parse . strip. uncomment, fvEmit = emit }
       emit SixsSbsMedV            = "sixs:sbsmedv"
       emit SixsSbsMedVFixDetector = "sixs:sbsmedvfixdetector"
       emit CristalK6C             = "cristal:k6c"
+      emit MarsFlyscan            = "mars:flyscan"
+
+surfaceOrientation :: FieldValue SurfaceOrientation
+surfaceOrientation = FieldValue { fvParse = parse . strip . uncomment, fvEmit = emit }
+  where
+    parse ::  Text -> Either String SurfaceOrientation
+    parse t
+      | t == emit SurfaceOrientationVertical = Right SurfaceOrientationVertical
+      | t == emit SurfaceOrientationHorizontal = Right SurfaceOrientationHorizontal
+      | otherwise = Left ("Unsupported " ++ unpack t ++ " surface orientation (vertical or horizontal)")
+
+    emit :: SurfaceOrientation -> Text
+    emit SurfaceOrientationVertical   = "vertical"
+    emit SurfaceOrientationHorizontal = "horizontal"
+
 
 projectionType :: FieldValue ProjectionType
 projectionType = FieldValue { fvParse = parse . strip . uncomment, fvEmit = emit }
@@ -290,11 +325,36 @@ pathAbsDir = FieldValue
   , fvEmit = pack . fromAbsDir
   }
 
-configRange :: (Num a, Read a, Show a, Typeable a) => FieldValue (ConfigRange a)
-configRange = listWithSeparator "-" number'
 
-configRangeP :: Integral a => Parser (ConfigRange a)
-configRangeP = ConfigRange <$> (decimal `sepBy` char '-')
+inputRangeP :: Parser InputRange
+inputRangeP = inputRangeFromToP <|> inputRangeP'
+  where
+    inputRangeFromToP :: Parser InputRange
+    inputRangeFromToP =  InputRangeFromTo
+                         <$> decimal <* char '-'
+                         <*> decimal
+
+    inputRangeP' :: Parser InputRange
+    inputRangeP' = InputRangeSingle <$> decimal
+
+configRange :: FieldValue ConfigRange
+configRange = FieldValue {fvParse = parse, fvEmit = emit }
+    where
+      parse :: Text -> Either String ConfigRange
+      parse = parseOnly configRangeP
+
+      emit :: ConfigRange -> Text
+      emit (ConfigRange is) = unwords $ map (pack . showInputRange) is
+
+      showInputRange :: InputRange -> String
+      showInputRange (InputRangeSingle f)   = printf "%d" f
+      showInputRange (InputRangeFromTo f t) = printf "%d-%d" f t
+
+configRangeP :: Parser ConfigRange
+configRangeP = ConfigRange <$> (inputRangeP `sepBy` many (satisfy isSep))
+    where
+      isSep :: Char -> Bool
+      isSep c = c == ' ' || c == ','
 
 detector :: FieldValue (Detector Hkl DIM2)
 detector = FieldValue
@@ -341,44 +401,56 @@ files c = do
     then throwM (NoDataFilesInTheGivenDirectory dir)
     else case c ^. binocularsInputInputRange of
            Just r  -> do
-             let fs'' = filter (isInConfigRange r) fs'
+             let tmpl = maybe "%05d" (unpack . unInputTmpl) (c ^. binocularsInputTmpl)
+             let fs'' = filter (isInConfigRange tmpl r) fs'
              if null fs''
              then throwM (NoFilesInRangeInTheGivenDirectory dir r)
              else return fs''
            Nothing -> return fs'
     where
       isHdf5 :: Path Abs File -> Bool
--- debian bulleyes (signature changed ???)
-#if MIN_VERSION_path(0, 7, 0)
-      isHdf5 p = case ((fileExtension p) :: Maybe [Char]) of
+      isHdf5 p = case (fileExtension p :: Maybe [Char]) of
                    Nothing    -> False
                    (Just ext) -> ext `elem` [".h5", ".nxs"]
--- debian buster
-#else
-      isHdf5 p = case (fileExtension p) of
-                   ""  -> False
-                   ext -> ext `elem` [".h5", ".nxs"]
-#endif
-      matchIndex :: Path Abs File -> Int -> Bool
-      matchIndex p n = printf "%04d" n `isInfixOf` toFilePath p
 
-      isInConfigRange :: ConfigRange Int -> Path Abs File -> Bool
-      isInConfigRange (ConfigRange []) _ = True
-      isInConfigRange (ConfigRange [from]) p = any (matchIndex p) [from]
-      isInConfigRange (ConfigRange [from, to]) p = any (matchIndex p) [from..to]
-      isInConfigRange (ConfigRange (from:to:_)) p = any (matchIndex p) [from..to]
+      matchIndex :: Path Abs File -> String -> Int -> Bool
+      matchIndex p tmpl n = printf tmpl n `isInfixOf` toFilePath p
+
+      isInInputRange :: Path Abs File -> String -> InputRange -> Bool
+      isInInputRange p tmpl (InputRangeSingle i) = any (matchIndex p tmpl) [i]
+      isInInputRange p tmpl (InputRangeFromTo from to) = any (matchIndex p tmpl) [from..to]
+
+      isInConfigRange :: String -> ConfigRange -> Path Abs File -> Bool
+      isInConfigRange _ (ConfigRange []) _    = True
+      isInConfigRange tmpl (ConfigRange rs) p = any (isInInputRange p tmpl) rs
+
 
 
 replace' :: Int -> Int -> DestinationTmpl -> FilePath
 replace' f t = unpack . replace "{last}" (pack . show $ t) . replace "{first}" (pack . show $ f) . unDestinationTmpl
 
-destination' :: ConfigRange Int -> DestinationTmpl -> FilePath
-destination' (ConfigRange [])          = replace' 0 0
-destination' (ConfigRange [from])      = replace' from from
-destination' (ConfigRange [from, to])  = replace' from to
-destination' (ConfigRange (from:to:_)) = replace' from to
+destination' :: ConfigRange -> DestinationTmpl -> FilePath
+destination' (ConfigRange rs) = replace' from to
+  where
+    (from,to) = hull rs
 
-combineWithCmdLineArgs :: BinocularsConfig -> Maybe (ConfigRange Int) -> BinocularsConfig
+    froms :: [Int]
+    froms = [ case r of
+                (InputRangeSingle f)   -> f
+                (InputRangeFromTo f _) -> f
+            | r <- rs]
+
+    tos :: [Int]
+    tos = [ case r of
+              (InputRangeSingle f)   -> f
+              (InputRangeFromTo _ t) -> t
+          | r <- rs]
+
+    hull :: [InputRange] -> (Int, Int)
+    hull [] = (0, 0)
+    hull _  = (minimum froms, maximum tos)
+
+combineWithCmdLineArgs :: BinocularsConfig -> Maybe ConfigRange -> BinocularsConfig
 combineWithCmdLineArgs c mr = case mr of
                                 Nothing  -> c
                                 (Just _) -> c{_binocularsInputInputRange = mr}
@@ -430,8 +502,8 @@ getMask c d = case _binocularsInputMaskmatrix c of
 getResolution :: MonadThrow m => BinocularsConfig -> Int -> m [Double]
 getResolution c n
   | Data.List.length res == 1 = return $ replicate n (head res)
-  | Data.List.length res == n = return $ res
-  | otherwise = throwM $ (ResolutionNotCompatibleWithProjectionNbOfCoordinates res) n
+  | Data.List.length res == n = return res
+  | otherwise = throwM $ ResolutionNotCompatibleWithProjectionNbOfCoordinates res n
   where
     res = _binocularsProjectionResolution c
 
