@@ -19,8 +19,6 @@ module Hkl.Binoculars.Projections
   , GeometryPath(..)
   , HklPath(..)
   , HklBinocularsException(..)
-  , InputHkl(..)
-  , InputQxQyQz(..)
   , QxQyQzPath(..)
   , SamplePath(..)
   , badAttenuation
@@ -29,22 +27,21 @@ module Hkl.Binoculars.Projections
   , spaceQxQyQz
   ) where
 
-import           Control.Exception                 (Exception)
-import           Data.Array.Repa                   (Array, extent, listOfShape,
-                                                    size)
-import           Data.Array.Repa.Index             (DIM1, DIM2, DIM3, Z)
-import           Data.Array.Repa.Repr.ForeignPtr   (F, toForeignPtr)
-import           Data.Text                         (Text)
-import           Data.Typeable                     (typeOf)
-import           Data.Word                         (Word16)
-import           Foreign.C.Types                   (CBool, CDouble (..),
-                                                    CSize (..))
-import           Foreign.ForeignPtr                (ForeignPtr, withForeignPtr)
-import           Foreign.Marshal.Array             (withArrayLen)
-import           Foreign.Ptr                       (Ptr, nullPtr)
-import           Numeric.Units.Dimensional.Prelude (Angle, Length)
+import           Control.Exception               (Exception)
+import           Data.Array.Repa                 (Array, extent, listOfShape,
+                                                  size)
+import           Data.Array.Repa.Index           (DIM1, DIM2, DIM3, Z)
+import           Data.Array.Repa.Repr.ForeignPtr (F, toForeignPtr)
+import           Data.Text                       (Text)
+import           Data.Typeable                   (typeOf)
+import           Data.Word                       (Word16)
+import           Foreign.C.Types                 (CBool, CDouble (..),
+                                                  CSize (..))
+import           Foreign.ForeignPtr              (withForeignPtr)
+import           Foreign.Marshal.Array           (withArrayLen)
+import           Foreign.Ptr                     (Ptr, nullPtr)
 
-import           Prelude                           hiding (drop, mapM)
+import           Prelude                         hiding (drop, mapM)
 
 import           Hkl.Binoculars.Common
 import           Hkl.Binoculars.Config
@@ -52,8 +49,9 @@ import           Hkl.C.Binoculars
 import           Hkl.C.Geometry
 import           Hkl.C.Sample
 import           Hkl.Detector
-import           Hkl.H5                            hiding (File)
-import           Hkl.Orphan                        ()
+import           Hkl.H5                          hiding (File)
+import           Hkl.Image
+import           Hkl.Orphan                      ()
 import           Hkl.Types
 
 --  Common
@@ -90,6 +88,8 @@ data GeometryPath
                            , geometryPathDelta      :: Hdf5Path DIM1 Double
                            }
   | GeometryPathFix { geometryPathWavelength :: Hdf5Path Z Double }
+  | GeometryPathMars { geometryPathAxes       :: [Hdf5Path DIM1 Double]
+                     }
   | GeometryPathMedH { geometryPathWavelength :: Hdf5Path Z Double
                      , geometryPathAxes       :: [Hdf5Path DIM1 Double]
                      }
@@ -118,6 +118,7 @@ data AttenuationPath
                       , attenuationOffset      :: Int
                       , attenuationCoefficient :: Double
                       }
+    | ApplyedAttenuationFactorPath { attenuationPath :: Hdf5Path DIM1 Float }
     | NoAttenuation
     deriving Show
 
@@ -133,40 +134,33 @@ instance Show QxQyQzPath where
 
 type Resolutions = [Double]
 
-data InputQxQyQz a =
-  InputQxQyQz { detector     :: Detector Hkl DIM2
-              , filename     :: InputFn
-              , h5dpath      :: a
-              , output       :: FilePath
-              , resolutions  :: [Double]
-              , centralPixel :: (Int, Int)  -- x, y
-              , sdd'         :: Length Double  -- sample to detector distance
-              , detrot'      :: Angle Double
-              , mask         :: Maybe Mask
-              }
-  deriving Show
-
 data DataFrameQxQyQz
     = DataFrameQxQyQz
       Int -- n
       Double -- attenuation
       Geometry -- geometry
-      (ForeignPtr Word16) -- image
+      Image -- image
     deriving Show
 
 {-# INLINE spaceQxQyQz #-}
-spaceQxQyQz :: Detector a DIM2 -> Array F DIM3 Double -> Resolutions -> Maybe Mask -> Space DIM3 -> DataFrameQxQyQz -> IO (DataFrameSpace DIM3)
-spaceQxQyQz det pixels rs mmask' space (DataFrameQxQyQz _ att g img) =
+spaceQxQyQz :: Detector a DIM2 -> Array F DIM3 Double -> Resolutions -> Maybe Mask -> SurfaceOrientation -> Space DIM3 -> DataFrameQxQyQz -> IO (DataFrameSpace DIM3)
+spaceQxQyQz det pixels rs mmask' surf space (DataFrameQxQyQz _ att g img) =
   withNPixels det $ \nPixels ->
-    withGeometry g $ \geometry ->
-    withForeignPtr (toForeignPtr pixels) $ \pix ->
-    withArrayLen rs $ \nr r ->
-    withPixelsDims pixels $ \ndim dims ->
-    withMaybeMask mmask' $ \ mask'' ->
-    withForeignPtr img $ \i ->
-    withForeignPtr (spaceHklPointer space) $ \pSpace -> do
-      {-# SCC "hkl_binoculars_space_q" #-} hkl_binoculars_space_q pSpace geometry i nPixels (CDouble att) pix (toEnum ndim) dims r (toEnum nr) mask''
-      return (DataFrameSpace img space att)
+  withGeometry g $ \geometry ->
+  withForeignPtr (toForeignPtr pixels) $ \pix ->
+  withArrayLen rs $ \nr r ->
+  withPixelsDims pixels $ \ndim dims ->
+  withMaybeMask mmask' $ \ mask'' ->
+  withForeignPtr (spaceHklPointer space) $ \pSpace -> do
+  case img of
+    (ImageInt32 fp) -> withForeignPtr fp $ \i -> do
+      {-# SCC "hkl_binoculars_space_q_int32_t" #-} hkl_binoculars_space_q_int32_t pSpace geometry i nPixels (CDouble att) pix (toEnum ndim) dims r (toEnum nr) mask'' (toEnum $ fromEnum surf)
+    (ImageWord16 fp) -> withForeignPtr fp $ \i -> do
+      {-# SCC "hkl_binoculars_space_q_uint16_t" #-} hkl_binoculars_space_q_uint16_t pSpace geometry i nPixels (CDouble att) pix (toEnum ndim) dims r (toEnum nr) mask'' (toEnum $ fromEnum surf)
+    (ImageWord32 fp) -> withForeignPtr fp $ \i -> do
+      {-# SCC "hkl_binoculars_space_q_uint32_t" #-} hkl_binoculars_space_q_uint32_t pSpace geometry i nPixels (CDouble att) pix (toEnum ndim) dims r (toEnum nr) mask'' (toEnum $ fromEnum surf)
+
+  return (DataFrameSpace img space att)
 
 -- SamplePath
 
@@ -189,20 +183,6 @@ data SamplePath
 data HklPath = HklPath QxQyQzPath SamplePath
                deriving Show
 
-data InputHkl a =
-  InputHkl { detector     :: Detector Hkl DIM2
-           , filename     :: InputFn
-           , h5dpath      :: a
-           , output       :: FilePath
-           , resolutions  :: [Double]
-           , centralPixel :: (Int, Int)  -- x, y
-           , sdd'         :: Length Double  -- sample to detector distance
-           , detrot'      :: Angle Double
-           , config       :: BinocularsConfig
-           , mask         :: Maybe Mask
-           }
-  deriving Show
-
 data DataFrameHkl a
     = DataFrameHkl DataFrameQxQyQz (Sample Triclinic)
       deriving Show
@@ -212,13 +192,18 @@ spaceHkl :: BinocularsConfig -> Detector b DIM2 -> Array F DIM3 Double -> Resolu
 spaceHkl config' det pixels rs mmask' space (DataFrameHkl (DataFrameQxQyQz _ att g img) samp) = do
   let sample' = overloadSampleWithConfig config' samp
   withNPixels det $ \nPixels ->
-      withGeometry g $ \geometry ->
-      withSample sample' $ \sample ->
-      withForeignPtr (toForeignPtr pixels) $ \pix ->
-      withArrayLen rs $ \nr r ->
-      withMaybeMask mmask' $ \ mask'' ->
-      withPixelsDims pixels $ \ndim dims ->
-      withForeignPtr img $ \i ->
-      withForeignPtr (spaceHklPointer space) $ \pSpace -> do
-        {-# SCC "hkl_binoculars_space_hkl" #-} hkl_binoculars_space_hkl pSpace geometry sample i nPixels (CDouble att) pix (toEnum ndim) dims r (toEnum nr) mask''
-        return (DataFrameSpace img space att)
+    withGeometry g $ \geometry ->
+    withSample sample' $ \sample ->
+    withForeignPtr (toForeignPtr pixels) $ \pix ->
+    withArrayLen rs $ \nr r ->
+    withMaybeMask mmask' $ \ mask'' ->
+    withPixelsDims pixels $ \ndim dims ->
+    withForeignPtr (spaceHklPointer space) $ \pSpace -> do
+    case img of
+      (ImageInt32 fp) -> withForeignPtr fp $ \i -> do
+        {-# SCC "hkl_binoculars_space_hkl_int32_t" #-} hkl_binoculars_space_hkl_int32_t pSpace geometry sample i nPixels (CDouble att) pix (toEnum ndim) dims r (toEnum nr) mask''
+      (ImageWord16 fp) -> withForeignPtr fp $ \i -> do
+        {-# SCC "hkl_binoculars_space_hkl_uint16_t" #-} hkl_binoculars_space_hkl_uint16_t pSpace geometry sample i nPixels (CDouble att) pix (toEnum ndim) dims r (toEnum nr) mask''
+      (ImageWord32 fp) -> withForeignPtr fp $ \i -> do
+        {-# SCC "hkl_binoculars_space_hkl_uint32_t" #-} hkl_binoculars_space_hkl_uint32_t pSpace geometry sample i nPixels (CDouble att) pix (toEnum ndim) dims r (toEnum nr) mask''
+    return (DataFrameSpace img space att)
