@@ -21,8 +21,6 @@ module Hkl.H5
     , closeDataset
     , closeFile
     , getArrayInBuffer
-    , get_image
-    , get_image'
     , get_position
     , get_position_new
     , get_ub
@@ -30,7 +28,7 @@ module Hkl.H5
     , datasetShape
     , nxEntries
     , nxEntries'
-    , openDataset
+    , openDataset'
     , openDatasetWithAttr
     , openH5
     , set_image
@@ -65,7 +63,8 @@ import           Bindings.HDF5.Core              (HSize (HSize),
                                                   iterOrderCode)
 import           Bindings.HDF5.Dataset           (Dataset, closeDataset,
                                                   createDataset,
-                                                  getDatasetSpace, openDataset,
+                                                  getDatasetSpace,
+                                                  getDatasetType, openDataset,
                                                   readDataset, readDatasetInto,
                                                   writeDataset)
 import           Bindings.HDF5.Dataspace         (Dataspace,
@@ -76,6 +75,7 @@ import           Bindings.HDF5.Dataspace         (Dataspace,
                                                   getSimpleDataspaceExtentNDims,
                                                   getSimpleDataspaceExtentNPoints,
                                                   selectHyperslab, selectNone)
+import           Bindings.HDF5.Datatype          (getTypeClass, getTypeSize)
 import           Bindings.HDF5.Datatype.Internal (NativeType, hdfTypeOf1,
                                                   nativeTypeOf)
 import           Bindings.HDF5.Error             (withErrorCheck_)
@@ -88,6 +88,7 @@ import           Bindings.HDF5.Link              (visitLinks)
 import           Bindings.HDF5.Object            (ObjectId, ObjectType (..),
                                                   closeObject, getObjectType,
                                                   openObject)
+import           Bindings.HDF5.PropertyList.DAPL (DAPL)
 import           Bindings.HDF5.PropertyList.DXPL (DXPL)
 import           Bindings.HDF5.Raw               (H5L_info_t, HErr_t (HErr_t),
                                                   HId_t (HId_t), h5d_read,
@@ -98,21 +99,18 @@ import           Control.Monad.Extra             (fromMaybeM)
 import           Data.Array.Repa                 (Array, Shape, extent,
                                                   linearIndex, listOfShape,
                                                   size)
-import           Data.Array.Repa.Repr.ForeignPtr (F, fromForeignPtr,
-                                                  toForeignPtr)
+import           Data.Array.Repa.Repr.ForeignPtr (F, toForeignPtr)
 import           Data.ByteString.Char8           (ByteString, pack, packCString,
                                                   unpack)
 import           Data.IORef                      (modifyIORef', newIORef,
                                                   readIORef)
 import           Data.Vector.Storable            (Storable, Vector, freeze,
                                                   head, unsafeFromForeignPtr0)
-import           Data.Vector.Storable.Mutable    (MVector (..), new, replicate)
+import           Data.Vector.Storable.Mutable    (new)
 import           Data.Word                       (Word16)
 import           Foreign.C.String                (CString)
 import           Foreign.C.Types                 (CInt (CInt))
-import           Foreign.ForeignPtr              (ForeignPtr, newForeignPtr,
-                                                  withForeignPtr)
-import           Foreign.Marshal.Alloc           (finalizerFree, mallocBytes)
+import           Foreign.ForeignPtr              (ForeignPtr, withForeignPtr)
 import           Foreign.Ptr                     (FunPtr, Ptr,
                                                   freeHaskellFunPtr)
 import           Foreign.Ptr.Conventions         (In (In), InOut (InOut),
@@ -159,27 +157,7 @@ createDataspaceFromShape sh = createSimpleDataspace [HSize (fromIntegral s) | s
 castToVector :: (Shape sh, Storable e) => Array F sh e -> Vector e
 castToVector arr = unsafeFromForeignPtr0 (toForeignPtr arr) (size . extent $ arr)
 
-get_image :: Shape sh => Detector a sh -> Dataset -> Int -> IO (Array F sh Word16)
-get_image det d n = withDataspace (getDatasetSpace d) $ \dataspace -> do
-      let s = shape det
-          h = (HSize (fromIntegral n), Nothing,  HSize 1, Nothing) : shapeAsRangeToHyperslab s
-      selectHyperslab dataspace Set h
-      withDataspace (createDataspaceFromShape s) $ \memspace -> do
-        data_out@(MVector _ fp) <- Data.Vector.Storable.Mutable.replicate (size s) (0 :: Word16)
-        readDatasetInto d (Just memspace) (Just dataspace) Nothing data_out
-        return $ fromForeignPtr s fp
-
-get_image' :: Shape sh => Detector a sh -> Dataset -> Int -> IO (ForeignPtr Word16)
-get_image' det d n = withDataspace (getDatasetSpace d) $ \dataspace -> do
-      let s = shape det
-          h = (HSize (fromIntegral n), Nothing,  HSize 1, Nothing) : shapeAsRangeToHyperslab s
-      selectHyperslab dataspace Set h
-      withDataspace (createDataspaceFromShape s) $ \memspace -> do
-        p <- mallocBytes (size s * 2)
-        readDatasetInto' d (Just memspace) (Just dataspace) Nothing p
-        newForeignPtr finalizerFree p
-
-getArrayInBuffer :: Shape sh => ForeignPtr Word16 -> Detector a sh -> Dataset -> Int -> IO (ForeignPtr Word16)
+getArrayInBuffer :: (NativeType t, Shape sh) => ForeignPtr t -> Detector a sh -> Dataset -> Int -> IO (ForeignPtr t)
 getArrayInBuffer fbuf det d n = withDataspace (getDatasetSpace d) $ \dataspace -> do
       let s = shape det
           h = (HSize (fromIntegral n), Nothing,  HSize 1, Nothing) : shapeAsRangeToHyperslab s
@@ -267,6 +245,18 @@ datasetShape d = withDataspace (getDatasetSpace d) getSimpleDataspaceExtent
 
 --  DataSet
 
+openDataset' :: Location l => l -> ByteString -> Maybe DAPL -> IO Dataset
+openDataset' loc n ml = do
+  ds <- openDataset loc n ml
+  s <- getDatasetSpace ds
+  np <- getSimpleDataspaceExtentNPoints s
+  e <- getSimpleDataspaceExtent s
+  t <- getDatasetType ds
+  c <- getTypeClass t
+  es <- getTypeSize t
+  -- print (n, c, es, np, e)
+  return ds
+
 withDataset :: IO Dataset -> (Dataset -> IO r) -> IO r
 withDataset a = bracket a closeDataset
 
@@ -336,16 +326,16 @@ openDatasetWithAttr loc attr value = do
       case t of
         DatasetObj -> do
           exist <- doesAttributeExist obj attr
-          case exist of
-            True -> withAttribute (openAttribute obj attr) $ \a -> do
+          if exist
+            then withAttribute (openAttribute obj attr) $ \a -> do
               c <- readAttributeStringASCII a
-              case c == value of
-                True -> do
-                  ds <- openDataset g n Nothing
-                  modifyIORef' state $ \_ -> (Just ds)
+              if c == value
+                then do
+                  ds <- openDataset' g n Nothing
+                  modifyIORef' state $ const (Just ds)
                   return $ HErr_t 1
-                False -> return $ HErr_t 0
-            False -> return $ HErr_t 0
+                else return $ HErr_t 0
+            else return $ HErr_t 0
         _          -> return $ HErr_t 0
   fromMaybeM (throwIO $ CanNotFindDatasetWithAttributContent attr value) (readIORef state)
 
@@ -413,9 +403,9 @@ withHdf5Path' :: Location l => l -> Hdf5Path sh e -> (Dataset -> IO r) -> IO r
 withHdf5Path' loc (H5RootPath subpath) f = withHdf5Path' loc subpath f
 withHdf5Path' loc (H5GroupPath n subpath) f = withGroup (openGroup loc n Nothing) $ \g -> withHdf5Path' g subpath f
 withHdf5Path' loc (H5GroupAtPath i subpath) f = withGroupAt loc i $ \g -> withHdf5Path' g subpath f
-withHdf5Path' loc (H5DatasetPath n) f = withDataset (openDataset loc n Nothing) f
+withHdf5Path' loc (H5DatasetPath n) f = withDataset (openDataset' loc n Nothing) f
 withHdf5Path' loc (H5DatasetPathAttr (a, c)) f = withDataset (openDatasetWithAttr loc a c) f
-withHdf5Path' loc (H5Or l r) f = (withHdf5Path' loc l f) <|> (withHdf5Path' loc r f)
+withHdf5Path' loc (H5Or l r) f = withHdf5Path' loc l f <|> withHdf5Path' loc r f
 
 withHdf5Path :: FilePath -> Hdf5Path sh e -> (Dataset -> IO r) -> IO r
 withHdf5Path fn path f = withH5File fn $ \fn' -> withHdf5Path' fn' path f
