@@ -28,7 +28,7 @@ import           Numeric.Units.Dimensional.Prelude (Length)
 type AbsDirPath = FilePath
 type SampleName = String
 
--- | Beamline
+--  Beamline
 
 data Beamline = Diffabs | Sixs
 
@@ -39,38 +39,38 @@ instance Show Beamline where
 beamlineUpper ∷ Beamline → String
 beamlineUpper b = [toUpper x | x ← show b]
 
--- | Engine
+--  Engine
 
 data Mode
   = Mode
-    String -- ^ name
-    [Parameter] -- ^ parameters of the @Mode@
+    String --  name
+    [Parameter] --  parameters of the @Mode@
   deriving (Show)
 
 data Engine
   = Engine
-    String -- ^ name
-    [Parameter] -- ^ pseudo axes values of the @Engine@
-    Mode -- ^ current Mode
+    String --  name
+    [Parameter] --  pseudo axes values of the @Engine@
+    Mode --  current Mode
   deriving (Show)
 
--- | Sample
+--  Sample
 
 data Sample a
-  = Sample
-    String -- ^ name of the sample
-    (Lattice a) -- ^ the lattice of the sample
-    Parameter -- ^ ux
-    Parameter -- ^ uy
-    Parameter -- ^ uz
+    = Sample { sampleName    :: SampleName
+             , sampleLattice :: Lattice a
+             , sampleUx      :: Parameter
+             , sampleUy      :: Parameter
+             , sampleUz      :: Parameter
+             }
   deriving (Show)
 
--- | Source
+--  Source
 
 type WaveLength = Length Double
 
 newtype Source = Source WaveLength deriving (Show)
 
--- | Trajectory
+--  Trajectory
 
 type Trajectory = [[Double]]
